@@ -1,4 +1,4 @@
-# Copyright 2020 Mechanics of Microstructures Group
+# Copyright 2021 Mechanics of Microstructures Group
 #    at The University of Manchester
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,14 +17,18 @@ import numpy as np
 
 from defdap import plotting
 
+from typing import Union, Tuple, List
 
 class Quat(object):
-    """Class used to define and perform operations on quaternions
+    """Class used to define and perform operations on quaternions. These
+    are interpreted in the passive sense.
 
     """
     __slots__ = ['quatCoef']
 
-    def __init__(self, *args):
+    def __init__(self,
+                 *args: Union[Tuple, list, np.ndarray],
+                 allow_southern: bool=False):
         """
         Construct a Quat object from 4 quat coefficients or an array of
         quat coefficients.
@@ -33,6 +37,8 @@ class Quat(object):
         ----------
         *args
             Variable length argument list.
+        allow_southern
+            if False, move quat to northern hemisphere.
 
         """
         # construct with array of quat coefficients
@@ -51,20 +57,24 @@ class Quat(object):
                             "quat coefficients")
 
         # move to northern hemisphere
-        if self.quatCoef[0] < 0:
+        if not allow_southern and self.quatCoef[0] < 0:
             self.quatCoef = self.quatCoef * -1
 
     @classmethod
-    def fromEulerAngles(cls, ph1, phi, ph2):
+    def fromEulerAngles(cls,
+                        ph1: float,
+                        phi: float,
+                        ph2: float
+                        ) -> 'Quat':
         """Create a quat object from 3 Bunge euler angles.
 
         Parameters
         ----------
-        ph1 : float
+        ph1
             First Euler angle, rotation around Z in radians.
-        phi : float
+        phi
             Second Euler angle, rotation around new X in radians.
-        ph2 : float
+        ph2
             Third Euler angle, rotation around new Z in radians.
 
         Returns
@@ -85,14 +95,18 @@ class Quat(object):
         return cls(quatCoef)
 
     @classmethod
-    def fromAxisAngle(cls, axis, angle):
-        """Create a quat object from an axis angle pair.
+    def fromAxisAngle(cls,
+                      axis: np.ndarray,
+                      angle: float
+                      ) -> 'Quat':
+        """Create a quat object from a rotation around an axis. This
+        creates a quaternion to represent the passive rotation (-ve axis).
 
         Parameters
         ----------
-        axis : np.ndarray
+        axis
             Axis that the rotation is applied around.
-        angle : float
+        angle
             Magnitude of rotation in radians.
 
         Returns
@@ -107,17 +121,18 @@ class Quat(object):
         # calculate quat coefficients
         quatCoef = np.zeros(4, dtype=float)
         quatCoef[0] = np.cos(angle / 2)
-        quatCoef[1:4] = np.sin(angle / 2) * axis
+        quatCoef[1:4] = -np.sin(angle / 2) * axis
 
         # call constructor
         return cls(quatCoef)
 
-    def eulerAngles(self):
+    def eulerAngles(self
+                    ) -> np.ndarray:
         """Calculate the Euler angle representation for this rotation.
 
         Returns
         -------
-        eulers : np.ndarray, shape 3
+        eulers : numpy.ndarray, shape 3
             Bunge euler angles (in radians).
 
         References
@@ -169,12 +184,13 @@ class Quat(object):
 
         return eulers
 
-    def rotMatrix(self):
+    def rotMatrix(self
+                  ) -> np.ndarray:
         """Calculate the rotation matrix representation for this rotation.
 
         Returns
         -------
-        rotMatrix : np.ndarray, shape (3, 3)
+        rotMatrix : numpy.ndarray, shape (3, 3)
             Rotation matrix.
 
         References
@@ -214,11 +230,16 @@ class Quat(object):
     def __str__(self):
         return self.__repr__()
 
-    def _plotIPF(self, direction, symGroup, **kwargs):
+    def _plotIPF(self,
+                 direction: np.ndarray,
+                 symGroup: str,
+                 **kwargs):
         Quat.plotIPF([self], direction, symGroup, **kwargs)
 
     # overload * operator for quaternion product and vector product
-    def __mul__(self, right):
+    def __mul__(self,
+                right: 'Quat',
+                allow_southern: bool = False):
         if isinstance(right, type(self)):   # another quat
             newQuatCoef = np.zeros(4, dtype=float)
             newQuatCoef[0] = (
@@ -230,22 +251,23 @@ class Quat(object):
                     right.quatCoef[0] * self.quatCoef[1:4] +
                     np.cross(self.quatCoef[1:4], right.quatCoef[1:4])
             )
-            return Quat(newQuatCoef)
-        raise TypeError()
+            return Quat(newQuatCoef, allow_southern=allow_southern)
 
-    # # overload % operator for dot product
-    # def __mod__(self, right):
-    def dot(self, right):
+        raise TypeError("{:} - {:}".format(type(self), type(right)))
+
+    def dot(self,
+            right: 'Quat'
+            ) -> float:
         """ Calculate dot product between two quaternions.
 
         Parameters
         ----------
-        right : defdap.quat.Quat
+        right
             Right hand quaternion.
 
         Returns
         -------
-        defdap.quat.Quat
+        float
             Dot product.
 
         """
@@ -254,27 +276,33 @@ class Quat(object):
         raise TypeError()
 
     # overload + operator
-    def __add__(self, right):
+    def __add__(self,
+                right: 'Quat'):
         if isinstance(right, type(self)):
             return Quat(self.quatCoef + right.quatCoef)
         raise TypeError()
 
     # overload += operator
-    def __iadd__(self, right):
+    def __iadd__(self,
+                 right: 'Quat'):
         if isinstance(right, type(self)):
             self.quatCoef += right.quatCoef
             return self
         raise TypeError()
 
     # allow array like setting/getting of components
-    def __getitem__(self, key):
+    def __getitem__(self,
+                    key: int):
         return self.quatCoef[key]
 
-    def __setitem__(self, key, value):
+    def __setitem__(self,
+                    key: int,
+                    value: float):
         self.quatCoef[key] = value
         return
 
-    def norm(self):
+    def norm(self
+             ) -> float:
         """Calculate the norm of the quaternion.
 
         Returns
@@ -285,7 +313,8 @@ class Quat(object):
         """
         return np.sqrt(np.dot(self.quatCoef[0:4], self.quatCoef[0:4]))
 
-    def normalise(self):
+    def normalise(self
+                  ) -> 'Quat':
         """ Normalise the quaternion (turn it into an unit quaternion).
 
         Returns
@@ -299,7 +328,8 @@ class Quat(object):
 
     # also the inverse if this is a unit quaternion
     @property
-    def conjugate(self):
+    def conjugate(self
+                  ) -> 'Quat':
         """Calculate the conjugate of the quaternion.
 
         Returns
@@ -310,31 +340,45 @@ class Quat(object):
         """
         return Quat(self[0], -self[1], -self[2], -self[3])
 
-    def transformVector(self, vector):
-        """Transforms vector by the quaternion. For EBSD quaterions this
-        is a transformation from sample space to crystal space. Perform
-        on conjugate of quaternion for crystal to sample.
+    def transformVector(self,
+                        vector: Union[Tuple, List, np.ndarray]
+                        ) -> np.ndarray:
+        """
+        Transforms vector by the quaternion. For passive EBSD quaterions
+        this is a transformation from sample space to crystal space.
+        Perform on conjugate of quaternion for crystal to sample. For a
+        quaternion representing a passive rotation from CS1 to CS2 and a
+        fixed vector V defined in CS1, this gives the coordinates
+        of V in CS2.
 
         Parameters
         ----------
-        vector : array_like, shape 3
+        vector : numpy.ndarray, shape 3 or equivalent
             Vector to transform.
 
         Returns
         -------
-        np.ndarray, shape 3
+        numpy.ndarray, shape 3
             Transformed vector.
 
         """
-        if isinstance(vector, np.ndarray) and vector.shape == (3,):
-            vectorQuat = Quat(0, vector[0], vector[1], vector[2])
-            vectorQuatTransformed = self * (vectorQuat * self.conjugate)
-            vectorTransformed = vectorQuatTransformed.quatCoef[1:4]
-            return vectorTransformed
+        if not isinstance(vector, (np.ndarray, list, tuple)):
+            raise TypeError("Vector must be a tuple, list or numpy array.")
+        if np.array(vector).shape != (3,):
+            raise TypeError("Vector must be length 3.")
 
-        raise TypeError("Vector must be a size 3 numpy array.")
+        vectorQuat = Quat(0, *vector)
+        vectorQuatTransformed = self.__mul__(
+            vectorQuat.__mul__(self.conjugate, allow_southern=True),
+            allow_southern=True
+        )
+        return vectorQuatTransformed.quatCoef[1:4]
 
-    def misOri(self, right, symGroup, returnQuat=0):
+    def misOri(self,
+               right: 'Quat',
+               symGroup: str,
+               returnQuat: int = 0
+               ) -> Tuple[float, 'Quat']:
         """
         Calculate misorientation angle between 2 orientations taking
         into account the symmetries of the crystal structure.
@@ -342,11 +386,11 @@ class Quat(object):
 
         Parameters
         ----------
-        right : defdap.quat.Quat
+        right
             Orientation to find misorientation to.
-        symGroup : str
+        symGroup
             Crystal type (cubic, hexagonal).
-        returnQuat : int
+        returnQuat
             What to return: 0 for minimum misorientation, 1 for
             symmetric equivalent with minimum misorientation, 2 for both.
 
@@ -355,7 +399,7 @@ class Quat(object):
         float
             Minimum misorientation.
         defdap.quat.Quat
-            Symmetric equivalent orientation with minimum  misorientation.
+            Symmetric equivalent orientation with minimum misorientation.
 
         """
         if isinstance(right, type(self)):
@@ -377,7 +421,9 @@ class Quat(object):
                 return minMisOri
         raise TypeError("Input must be a quaternion.")
 
-    def misOriAxis(self, right):
+    def misOriAxis(self,
+                   right: 'Quat'
+                   ) -> np.ndarray:
         """
         Calculate misorientation axis between 2 orientations. This
         does not consider symmetries of the crystal structure.
@@ -397,42 +443,49 @@ class Quat(object):
             Dq = right * self.conjugate
             Dq = Dq.quatCoef
             misOriAxis = 2 * Dq[1:4] * np.arccos(Dq[0]) / np.sqrt(1 - Dq[0]**2)
-
             return misOriAxis
         raise TypeError("Input must be a quaternion.")
 
-    def plotIPF(self, direction, symGroup, projection=None,
-                plot=None, fig=None, ax=None, makeInteractive=False,
-                plotColourBar=False, clabel="",
-                markerColour=None, markerSize=40, **kwargs):
+    def plotIPF(self,
+                direction: np.ndarray,
+                symGroup: str,
+                projection:  str = None,
+                plot: 'plotting.Plot' = None,
+                fig: 'matplotlib.figure.Figure' = None,
+                ax: 'matplotlib.axes.Axes' = None,
+                makeInteractive: bool = False,
+                markerColour: Union[List[str], str] = None,
+                markerSize: float = 40,
+                **kwargs
+                ) -> 'plotting.PolePlot':
         """
         Plot IPF of orientation, with relation to specified sample direction.
 
         Parameters
         ----------
-        direction : np.ndarray
+        direction
             Sample reference direction for IPF.
-        symGroup : str
+        symGroup
             Crystal type (cubic, hexagonal).
-        projection : str
+        projection
              Projection to use. Either string (stereographic or lambert)
              or a function.
-        plot : defdap.plotting.Plot
+        plot
             Defdap plot to plot on.
-        fig : matplotlib.figure.Figure
+        fig
             Figure to plot on, if not provided the current
             active axis is used.
-        ax : matplotlib.axes.Axes
+        ax
             Axis to plot on, if not provided the current
             active axis is used.
-        makeInteractive : bool
+        makeInteractive
             If true, make the plot interactive.
-        markerColour : str
+        markerColour: str or list of str
             Colour of markers (only used for half and half colouring,
-            otherwise us argument c).
-        markerSize : int
+            otherwise use argument c).
+        markerSize
             Size of markers (only used for half and half colouring,
-            otherwise us argument s).
+            otherwise use argument s).
         kwargs
             All other arguments are passed to :func:`defdap.plotting.PolePlot.addPoints`.
 
@@ -457,33 +510,31 @@ class Quat(object):
                        markerColour=markerColour, markerSize=markerSize,
                        **plotParams)
 
-        if plotColourBar:
-            plot.addColourBar(clabel)
-
         return plot
 
-    def plotUnitCell(self, symGroup, cOverA=None, OI=True,
-                     plot=None, fig=None, ax=None, makeInteractive=False,
+    def plotUnitCell(self,
+                     crystalStructure: 'defdap.crystal.CrystalStructure',
+                     OI: bool = True,
+                     plot: 'plotting.CrystalPlot' = None,
+                     fig: 'matplotlib.figure.Figure' = None,
+                     ax: 'matplotlib.axes.Axes' = None,
+                     makeInteractive: bool = False,
                      **kwargs):
         """Plots a unit cell.
 
         Parameters
         ----------
-        symGroup : str
-            Crystal type, hexagonal or cubic.
-        cOverA : float
-            C over a ratio for hexagonal.
-        OI : bool
+        crystalStructure
+            Crystal structure.
+        OI
             True if using oxford instruments system.
-        plot : defdap.plotting.CrystalPlot
+        plot
             Plot object to plot to.
-        fig : matplotlib.figure.Figure
-            Figure to plot on, if not provided the current
-            active axis is used.
-        ax : matplotlib.axes.Axes
-            Axis to plot on, if not provided the current
-            active axis is used.
-        makeInteractive : bool
+        fig
+            Figure to plot on, if not provided the current active axis is used.
+        ax
+            Axis to plot on, if not provided the current active axis is used.
+        makeInteractive
             True to make the plot interactive.
         kwargs
             All other arguments are passed to :func:`defdap.plotting.CrystalPlot.addVerts`.
@@ -493,74 +544,26 @@ class Quat(object):
         plotParams = {}
         plotParams.update(kwargs)
 
-        if symGroup is None:
-            raise ValueError("symGroup must be specified")
+        # TODO: most of this should be moved to either the crystal or plotting module
 
-        quat = self
-
-        if symGroup == 'hexagonal':
-            if cOverA is None:
-                raise ValueError("cOverA must be specified for hcp")
-
-            szFac = 0.2
-            sqrt3over2 = np.sqrt(3) / 2
-            cOverA /= 2
-            vert = np.array([
-                [1, 0, -cOverA],
-                [0.5, sqrt3over2, -cOverA],
-                [-0.5, sqrt3over2, -cOverA],
-                [-1, 0, -cOverA],
-                [-0.5, -sqrt3over2, -cOverA],
-                [0.5, -sqrt3over2, -cOverA],
-                [1, 0, cOverA],
-                [0.5, sqrt3over2, cOverA],
-                [-0.5, sqrt3over2, cOverA],
-                [-1, 0, cOverA],
-                [-0.5, -sqrt3over2, cOverA],
-                [0.5, -sqrt3over2, cOverA]
-            ])
-            faces = [
-                [0, 1, 2, 3, 4, 5],
-                [6, 7, 8, 9, 10, 11],
-                [0, 6, 7, 1],
-                [1, 7, 8, 2],
-                [2, 8, 9, 3],
-                [3, 9, 10, 4],
-                [4, 10, 11, 5],
-                [5, 11, 6, 0]
-            ]
+        vert = crystalStructure.vertices
+        faces = crystalStructure.faces
 
+        if crystalStructure.name == 'hexagonal':
+            szFac = 0.18
             if OI:
                 # Add 30 degrees to phi2 for OI
-                eulerAngles = quat.eulerAngles()
+                eulerAngles = self.eulerAngles()
                 eulerAngles[2] += np.pi / 6
-                quat = Quat.fromEulerAngles(*eulerAngles)
+                gg = Quat.fromEulerAngles(*eulerAngles).rotMatrix().T
+            else:
+                gg = quatToPlot.rotMatrix().T
 
-        elif symGroup == 'cubic':
-            szFac = 0.3
-            vert = np.array([
-                [-0.5, -0.5, -0.5],
-                [0.5, -0.5, -0.5],
-                [0.5, 0.5, -0.5],
-                [-0.5, 0.5, -0.5],
-                [-0.5, -0.5, 0.5],
-                [0.5, -0.5, 0.5],
-                [0.5, 0.5, 0.5],
-                [-0.5, 0.5, 0.5]
-            ])
-            faces = [
-                [0, 1, 2, 3],
-                [4, 5, 6, 7],
-                [0, 1, 5, 4],
-                [1, 2, 6, 5],
-                [2, 3, 7, 6],
-                [3, 0, 4, 7]
-            ]
-        else:
-            raise ValueError("Only cubic and hexagonal supported")
+        elif crystalStructure.name == 'cubic':
+            szFac = 0.25
+            gg = quatToPlot.rotMatrix().T
 
         # Rotate the lattice cell points
-        gg = quat.rotMatrix().T
         pts = np.matmul(gg, vert.T).T * szFac
 
         # Plot unit cell
@@ -572,6 +575,13 @@ class Quat(object):
             plot = plotting.CrystalPlot(
                 ax=ax, fig=fig, makeInteractive=makeInteractive
             )
+
+        plot.ax.set_xlim3d(-0.15, 0.15)
+        plot.ax.set_ylim3d(-0.15, 0.15)
+        plot.ax.set_zlim3d(-0.15, 0.15)
+        plot.ax.view_init(azim=270, elev=90)
+        plot.ax._axis3don = False
+
         plot.addVerts(planes, **plotParams)
 
         return plot
@@ -579,12 +589,13 @@ class Quat(object):
 # Static methods
 
     @staticmethod
-    def createManyQuats(eulerArray):
+    def createManyQuats(eulerArray: np.ndarray
+                        ) -> np.ndarray:
         """Create a an array of quats from an array of Euler angles.
 
         Parameters
         ----------
-        eulerArray : numpy.ndarray
+        eulerArray
             Array of Bunge Euler angles of shape 3 x n x ... x m.
 
         Returns
@@ -613,21 +624,79 @@ class Quat(object):
         return quats
 
     @staticmethod
-    def calcSymEqvs(quats, symGroup, dtype=np.float):
+    def multiplyManyQuats(quats: List['Quat'],
+                          right: 'Quat'
+                          ) -> List['Quat']:
+        """ Multiply all quats in a list of quats, by a single quat.
+
+        Parameters
+        ----------
+        quats
+            List of quats to be operated on.
+        right
+            Single quaternion to multiply with the list of quats.
+
+        Returns
+        -------
+        list(defdap.quat.Quat)
+
+        """
+        quatArray = np.array([q.quatCoef for q in quats])
+
+        tempArray = np.zeros((len(quatArray),4), dtype=float)
+        tempArray[...,0] = ((quatArray[...,0] * right.quatCoef[0]) -
+                             np.dot(quatArray[...,1:4], right.quatCoef[1:4]))
+
+        tempArray[...,1:4] = ((quatArray[...,0,None] * right.quatCoef[None,1:4]) +
+                              (right.quatCoef[0] * quatArray[...,1:4]) +
+                              np.cross(quatArray[...,1:4], right.quatCoef[1:4]))
+
+        return [Quat(coefs) for coefs in tempArray]
+
+    @staticmethod
+    def extract_quat_comps(quats: np.ndarray):
+        """Return a NumPy array of the provided quaternion components
+
+        Input quaternions may be given as a list of Quat objects or any iterable
+        whose items have 4 components which map to the quaternion.
+
+        Parameters
+        ----------
+        quats : numpy.ndarray(defdap.quat.Quat)
+            A list of Quat objects to return the components of
+
+        Returns
+        -------
+        numpy.ndarray
+            Array of quaternion components, shape (4, ..)
+
+        """
+        quats = np.array(quats)
+        quat_comps = np.empty((4,) + quats.shape)
+        for idx in np.ndindex(quats.shape):
+            quat_comps[(slice(None),) + idx] = quats[idx].quatCoef
+
+        return quat_comps
+
+    @staticmethod
+    def calcSymEqvs(quats: np.ndarray,
+                    symGroup: str,
+                    dtype: type=np.float
+                    ) -> np.ndarray:
         """Calculate all symmetrically equivalent quaternions of given quaternions.
 
         Parameters
         ----------
         quats : numpy.ndarray(defdap.quat.Quat)
             Array of quat objects.
-        symGroup : str
+        symGroup
             Crystal type (cubic, hexagonal).
-        dtype : numpy.dtype
+        dtype
             Data type used for calculation, defaults to np.float.
 
         Returns
         -------
-        quatComps: np.ndarray, shape: (numSym x 4 x numQuats)
+        quatComps: numpy.ndarray, shape: (numSym x 4 x numQuats)
             Array containing all symmetrically equivalent quaternion components of input quaternions.
 
         """
@@ -635,7 +704,7 @@ class Quat(object):
         quatComps = np.empty((len(syms), 4, len(quats)), dtype=dtype)
 
         # store quat components in array
-        quatComps[0, :, :] = np.asarray([quat.quatCoef for quat in quats]).T
+        quatComps[0] = Quat.extract_quat_comps(quats)
 
         # calculate symmetrical equivalents
         for i, sym in enumerate(syms[1:], start=1):
@@ -659,14 +728,15 @@ class Quat(object):
         return quatComps
 
     @staticmethod
-    def calcAverageOri(quatComps):
+    def calcAverageOri(quatComps: np.ndarray
+                       ) -> 'Quat':
         """Calculate the average orientation of given quats.
 
         Parameters
         ----------
         quatComps : numpy.ndarray
             Array containing all symmetrically equivalent quaternion components of given quaternions
-            (shape: numSym x 4 x numQuats), can be calculated with Quat.calcSymEqvs.
+            (shape: numSym x 4 x numQuats), can be calculated with :func:`Quat.calcSymEqvs`.
 
         Returns
         -------
@@ -696,15 +766,17 @@ class Quat(object):
         return avOri
 
     @staticmethod
-    def calcMisOri(quatComps, refOri):
+    def calcMisOri(quatComps: np.ndarray,
+                   refOri: 'Quat'
+                   ) -> Tuple[np.ndarray, 'Quat']:
         """Calculate the misorientation between the quaternions and a reference quaternion.
 
         Parameters
         ----------
-        quatComps : numpy.ndarray
+        quatComps
             Array containing all symmetrically equivalent quaternion components of given quaternions
-            (shape: numSym x 4 x numQuats), can be calculated from quats with Quat.calcSymEqvs.
-        refOri : defdap.quat.Quat
+            (shape: numSym x 4 x numQuats), can be calculated from quats with :func:`Quat.calcSymEqvs` .
+        refOri
             Reference orientation.
 
         Returns
@@ -732,7 +804,9 @@ class Quat(object):
         return minMisOris, minQuatComps
 
     @staticmethod
-    def polarAngles(x, y, z):      # spherical coordinates as per Wikipedia
+    def polarAngles(x: np.ndarray,
+                    y: np.ndarray,
+                    z: np.ndarray):      # spherical coordinates as per Wikipedia
         """Convert Cartesian coordinates to polar coordinates, for an unit vector.
 
         Parameters
@@ -761,7 +835,11 @@ class Quat(object):
         return alpha, beta
 
     @staticmethod
-    def calcIPFcolours(quats, direction, symGroup, dtype=np.float32):
+    def calcIPFcolours(quats: np.ndarray,
+                       direction: np.ndarray,
+                       symGroup: str,
+                       dtype: type=np.float32
+                       ) -> np.ndarray:
         """
         Calculate the RGB colours, based on the location of the given quats
         on the fundamental region of the IPF for the sample direction specified.
@@ -770,11 +848,11 @@ class Quat(object):
         ----------
         quats : numpy.ndarray(defdap.quat.Quat)
             Array of quat objects.
-        direction : numpy.ndarray
+        direction
             Direction in sample space.
-        symGroup : str
+        symGroup
             Crystal type (cubic, hexagonal).
-        dtype: numpy.dtype
+        dtype
             Data type to use for calculation.
 
         Returns
@@ -787,8 +865,8 @@ class Quat(object):
         Stephen Cluff (BYU) - IPF_rgbcalc.m subroutine in OpenXY
         https://github.com/BYU-MicrostructureOfMaterials/OpenXY/blob/master/Code/PlotIPF.m
 
-        """
 
+        """
         numQuats = len(quats)
 
         alphaFund, betaFund = Quat.calcFundDirs(
@@ -865,26 +943,29 @@ class Quat(object):
         return rgb
 
     @staticmethod
-    def calcFundDirs(quats, direction, symGroup, dtype=np.float):
+    def calcFundDirs(quats: np.ndarray,
+                     direction: np.ndarray,
+                     symGroup: str,
+                     dtype: type = np.float):
         """
         Transform the sample direction to crystal coords based on the quats
         and find the ones in the fundamental sector of the IPF.
 
         Parameters
         ----------
-        quats : array_like(defdap.quat.Quat)
+        quats: array_like(defdap.quat.Quat)
             Array of quat objects.
-        direction: numpy.ndarray
+        direction
             Direction in sample space.
-        symGroup : str
+        symGroup
             Crystal type (cubic, hexagonal).
-        dtype: numpy.dtype
+        dtype
             Data type to use for calculation.
 
         Returns
         -------
         float, float
-            inclination angle and azimuthal angle (around z axis from x in anticlockwise as per ISO).
+            inclination angle and azimuthal angle (around z axis from x in anticlockwise).
 
         """
         # convert direction to float array
@@ -990,8 +1071,10 @@ class Quat(object):
         return alphaFund, betaFund
 
     @staticmethod
-    def symEqv(symGroup):
+    def symEqv(symGroup: str
+               ) -> List['Quat']:
         """Returns all symmetric equivalents for a given crystal type.
+        LEGACY: move to use symmetries defined in crystal structures
 
         Parameters
         ----------
@@ -1000,71 +1083,14 @@ class Quat(object):
 
         Returns
         -------
-        list(defdap.quat.Quat), shape (numQuats x 4)
+        list of defdap.quat.Quat
             Symmetrically equivalent quats.
 
         """
-        overRoot2 = np.sqrt(2) / 2
-        sqrt3over2 = np.sqrt(3) / 2
-
-        # from Pete Bate's fspl_orir.f90 code
-        # checked for consistency with mtex
-        qsym = [
-            # identity - this should always be returned as the first symmetry
-            Quat(1.0, 0.0, 0.0, 0.0),
-
-            # cubic tetrads(100)
-            Quat(overRoot2, overRoot2, 0.0, 0.0),
-            Quat(0.0, 1.0, 0.0, 0.0),
-            Quat(overRoot2, -overRoot2, 0.0, 0.0),
-
-            Quat(overRoot2, 0.0, overRoot2, 0.0),
-            Quat(0.0, 0.0, 1.0, 0.0),
-            Quat(overRoot2, 0.0, -overRoot2, 0.0),
-
-            Quat(overRoot2, 0.0, 0.0, overRoot2),
-            Quat(0.0, 0.0, 0.0, 1.0),
-            Quat(overRoot2, 0.0, 0.0, -overRoot2),
-
-            # cubic dyads (110)
-            Quat(0.0, overRoot2, overRoot2, 0.0),
-            Quat(0.0, -overRoot2, overRoot2, 0.0),
-
-            Quat(0.0, overRoot2, 0.0, overRoot2),
-            Quat(0.0, -overRoot2, 0.0, overRoot2),
-
-            Quat(0.0, 0.0, overRoot2, overRoot2),
-            Quat(0.0, 0.0, -overRoot2, overRoot2),
-
-            # cubic triads (111)
-            Quat(0.5, 0.5, 0.5, 0.5),
-            Quat(0.5, -0.5, -0.5, -0.5),
-
-            Quat(0.5, -0.5, 0.5, 0.5),
-            Quat(0.5, 0.5, -0.5, -0.5),
-
-            Quat(0.5, 0.5, -0.5, 0.5),
-            Quat(0.5, -0.5, 0.5, -0.5),
-
-            Quat(0.5, 0.5, 0.5, -0.5),
-            Quat(0.5, -0.5, -0.5, 0.5),
-
-            # hexagonal hexads
-            Quat(sqrt3over2, 0.0, 0.0, 0.5),
-            Quat(0.5, 0.0, 0.0, sqrt3over2),
-            Quat(0.5, 0.0, 0.0, -sqrt3over2),
-            Quat(sqrt3over2, 0.0, 0.0, -0.5),
-
-            # hexagonal diads
-            Quat(0.0, -0.5, -sqrt3over2, 0.0),
-            Quat(0.0, 0.5, -sqrt3over2, 0.0),
-            Quat(0.0, sqrt3over2, -0.5, 0.0),
-            Quat(0.0, -sqrt3over2, -0.5, 0.0)
-        ]
-
-        if symGroup == 'cubic':
-            return qsym[0:24]
-        elif symGroup == 'hexagonal':
-            return [qsym[0], qsym[2], qsym[5], qsym[8]] + qsym[-8:32]
-        else:
-            return [qsym[0]]
+        # Dirty fix to stop circular dependency
+        from defdap.crystal import crystalStructures
+        try:
+            return crystalStructures[symGroup].symmetries
+        except KeyError:
+            # return just identity if unknown structure
+            return [Quat(1.0, 0.0, 0.0, 0.0)]
